<?php
/**
 * @file
 * Authcache personalized blocks administrative interface.
 */

/**
 * Authcache personalized blocks overview page.
 */
function authcache_block_admin_blocks() {
  drupal_set_title(t('Authcache personalized blocks'));
  drupal_add_css(drupal_get_path('module', 'authcache_block') . '/authcache_block.admin.css');

  $page['header']['#markup'] = '<p>' . t('!blocks may be configured to load using Ajax or ESI. This is useful for dynamic or user-centric content.', array('!blocks' => l(t('Blocks'), 'admin/structure/block'))) . '</p>';

  module_load_include('inc', 'block', 'block.admin');
  $rows = array();
  foreach (list_themes() as $theme) {
    if ($theme->status) {
      $blocks = block_admin_display_prepare_blocks($theme->name);

      $region_rows = array();
      foreach ($blocks as $block) {
        $block_id = $block['module'] . '-' . $block['delta'];
        $options = variable_get('authcache_block-' . $block_id, array());

        if (empty($options['enabled'])) {
          continue;
        }

        $lifetime = format_interval($options['settings']['lifespan']);

        $region = $block['region'];

        if ($block['region'] == -1) {
          $region = '<em>' . t('block disabled') . '</em>';
        }

        $region_rows[] = array(
          l($block['info'], 'admin/structure/block/manage/' . $block['module'] . '/' . $block['delta'] . '/configure', array('query' => array('destination' => 'admin/config/system/authcache/blocks'))),
          $lifetime,
          $region,
        );

        if (!empty($region_rows)) {
          $rows[] = array(
            'data' => array(
              array(
                'data' => check_plain($theme->info['name']),
                'colspan' => 3,
              ),
            ),
            'class' => array('region-title'),
          );
          $rows = array_merge($rows, $region_rows);
        }
      }
    }
  }

  if (!empty($rows)) {
    $header = array(t('Block'), t('Cache Lifetime'), t('Region'));
    $page['table']['#markup'] = theme('table', array(
      'header' => $header,
      'rows' => $rows,
      'attributes' => array(
        'id' => 'authcache-blocks',
      ),
    ));
  }

  $page['footer']['#markup'] = '<em>' . t('To add a block to the list of personalized blocks, visit the !blocks page and click "configure" next to the block in question.', array('!blocks' => l(t('Blocks'), 'admin/structure/block'))) . '</em>';
  return $page;
}
