<?php
/**
 * @file
 * Defines authcache ajax command for retrieving debug information about the
 * current page.
 */

class AuthcacheDebugInfoSetting implements AuthcacheP13nSettingInterface, AuthcacheP13nSettingValidatorInterface, AuthcacheP13nSettingAccessInterface {
  /**
   * Ensure that the parameters consist of exactly one string.
   */
  public function validate($params) {
    if (empty($params) || !is_string($params)) {
      throw new AuthcacheP13nRequestInvalidInput();
    }

    return $params;
  }

  /**
   * Check whether the user has access to the given setting.
   */
  public function check($account, $params) {
    include_once dirname(__FILE__) . '/../authcache_debug.module';

    return authcache_debug_access();
  }

  /**
   * Return debug information for the given page request.
   */
  public function get($params) {
    $cache = cache_get($params, 'cache_authcache_debug');

    if ($cache) {
      return $cache->data;
    }
    else {
      throw new AuthcacheP13nRequestNotFound();
    }
  }
}
