<?php
/**
 * @file
 * Defines authcache P13n default request registry implementation.
 */

/**
 * Default implementation of AuthcacheP13nRequestRegistry.
 */
class AuthcacheP13nDefaultRequestRegistry implements AuthcacheP13nRequestRegistryInterface {
  /**
   * Return a map (path -> request-id).
   */
  public function routes() {
    return variable_get('authcache_p13n_routes', array());
  }

  /**
   * Given a path return an appropriate request id.
   */
  public function resolve($path) {
    $routes = $this->routes();

    if (empty($routes[$path])) {
      throw new AuthcacheP13nRequestNotFound();
    }

    return $routes[$path];
  }

  /**
   * Return the handler for the given request id.
   */
  public function get($request_id) {
    $handler = NULL;

    $cache = cache_get($request_id, 'cache_authcache_p13n');
    if ($cache) {
      $handler = $cache->data;
    }
    elseif (variable_get('authcache_p13n_default_registry_autorebuild', TRUE)) {
      // Make sure we're fully bootstrapped.
      drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);

      // Rebuild request registry and find handler for request.
      $requests = $this->rebuild();
      if (!empty($requests[$request_id])) {
        $handler = $requests[$request_id];
      }
    }

    if (!$handler) {
      watchdog('authcache_p13n', 'Default registry: Failed to find handler for request id %r', array('%r' => $request_id), WATCHDOG_ERROR);
      throw new AuthcacheP13nRequestInvalidInput();
    }

    return $handler;
  }

  /**
   * Rebuild request handlers.
   */
  public function rebuild() {
    // Collect all request definitions.
    $requests = authcache_p13n_request_collect_info();

    foreach ($requests as $request_id => $request) {
      cache_set($request_id, $request, 'cache_authcache_p13n');
    }

    $routes = authcache_p13n_request_collect_routes($requests);
    variable_set('authcache_p13n_routes', $routes);

    return $requests;
  }
}
